from Network import Network
from Model import Model, Constants
from AI import AI
import json
from threading import Thread, Timer
from queue import Queue


class Controller():
    def __init__(self, settings_file):
        self.settings_file = settings_file
        self.conf = {}
        self.network = None
        self.queue = Queue()
        self.model = Model(self.queue)
        self.client = AI()

    def start(self):
        self.read_settings()
        self.network = Network(ip=self.conf["ip"], port=self.conf["port"],
                               token=self.conf["token"], message_handler=self.handle_message)
        while not self.network.is_connected:
            self.network.connect()

    def terminate(self):
        print("finished!")
        self.network.close()

    def read_settings(self):
        with open(self.settings_file) as file:
            self.conf = json.loads(file.read())

    def handle_message(self, message):
        if message['name'] == 'init':
            self.model.handle_init_message(message)
        elif message['name'] == 'turn':
            self.model.handle_turn_message(message)
            self.do_turn()
        elif message['name'] == 'shutdown':
            self.terminate()

    def do_turn(self):

        def run():

            def timeout():
                events = []
                while not self.queue.qsize() == 0:
                    events.append(self.queue.get())
                    self.queue.task_done()
                message = {
                    Constants.KEY_NAME: Constants.MESSAGE_TYPE_EVENT,
                    Constants.KEY_ARGS: [event for event in events]
                }
                self.network.send(message)

            Timer(self.model.turn_timeout, timeout).start()
            self.client.do_turn(self.model.world)

        tr = Thread(target=run, daemon=True)
        tr.start()

c = Controller("connection.conf")
c.start()